# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["FlowGetMetricsResponse", "Metrics", "MetricsMetricsByUser", "Workflow"]


class MetricsMetricsByUser(BaseModel):
    completed: Optional[int] = None

    failed: Optional[int] = None

    in_progress: Optional[int] = None

    total: Optional[int] = None


class Metrics(BaseModel):
    avg_duration_ms: Optional[float] = None

    completed_workflows: Optional[int] = None

    failed_workflows: Optional[int] = None

    in_progress_workflows: Optional[int] = None

    metrics_by_user: Optional[Dict[str, MetricsMetricsByUser]] = None

    total_workflows: Optional[int] = None


class Workflow(BaseModel):
    id: Optional[str] = None

    end_timestamp: Optional[datetime] = None

    org_id: Optional[str] = None

    start_timestamp: Optional[datetime] = None

    status: Optional[Literal["completed", "failed", "in_progress"]] = None

    user_id: Optional[str] = None

    workflow_id: Optional[str] = None


class FlowGetMetricsResponse(BaseModel):
    metrics: Optional[Metrics] = None

    workflows: Optional[List[Workflow]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .db import (
    DBResource,
    AsyncDBResource,
    DBResourceWithRawResponse,
    AsyncDBResourceWithRawResponse,
    DBResourceWithStreamingResponse,
    AsyncDBResourceWithStreamingResponse,
)
from .flows import (
    FlowsResource,
    AsyncFlowsResource,
    FlowsResourceWithRawResponse,
    AsyncFlowsResourceWithRawResponse,
    FlowsResourceWithStreamingResponse,
    AsyncFlowsResourceWithStreamingResponse,
)
from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)

__all__ = [
    "DBResource",
    "AsyncDBResource",
    "DBResourceWithRawResponse",
    "AsyncDBResourceWithRawResponse",
    "DBResourceWithStreamingResponse",
    "AsyncDBResourceWithStreamingResponse",
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "FlowsResource",
    "AsyncFlowsResource",
    "FlowsResourceWithRawResponse",
    "AsyncFlowsResourceWithRawResponse",
    "FlowsResourceWithStreamingResponse",
    "AsyncFlowsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["DBExecuteQueryResponse"]


class DBExecuteQueryResponse(BaseModel):
    data: Optional[List[object]] = None

    execution_time: Optional[int] = FieldInfo(alias="executionTime", default=None)
    """Query execution time in milliseconds"""

    query: Optional[str] = None
    """The executed SQL query"""

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DBDeleteRecordsParams"]


class DBDeleteRecordsParams(TypedDict, total=False):
    table: Required[str]
    """Table name to delete from"""

    where: Required[object]
    """Where conditions"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["DBDeleteRecordsResponse"]


class DBDeleteRecordsResponse(BaseModel):
    count: Optional[int] = None
    """Number of records deleted"""

    data: Optional[List[object]] = None

    message: Optional[str] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DBProcessNlQueryParams"]


class DBProcessNlQueryParams(TypedDict, total=False):
    question: Required[str]
    """Natural language question"""

    context: object
    """Additional context for the query"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["DBProcessNlQueryResponse"]


class DBProcessNlQueryResponse(BaseModel):
    data: Optional[List[object]] = None

    message: Optional[str] = None

    sql: Optional[str] = None
    """The generated SQL query"""

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DBInsertRecordParams"]


class DBInsertRecordParams(TypedDict, total=False):
    data: Required[object]
    """Data to insert"""

    table: Required[str]
    """Table name to insert into"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from worqhat import Worqhat, AsyncWorqhat
from tests.utils import assert_matches_type
from worqhat.types import (
    FlowGetMetricsResponse,
    FlowTriggerWithFileResponse,
    FlowTriggerWithPayloadResponse,
)
from worqhat._utils import parse_date

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFlows:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_metrics(self, client: Worqhat) -> None:
        flow = client.flows.get_metrics()
        assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_metrics_with_all_params(self, client: Worqhat) -> None:
        flow = client.flows.get_metrics(
            end_date=parse_date("2025-07-24"),
            start_date=parse_date("2025-07-01"),
            status="completed",
            user_id="member-test-2f9b9a4f-5898-4e7a-8f26-e60cea49ae31",
        )
        assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_metrics(self, client: Worqhat) -> None:
        response = client.flows.with_raw_response.get_metrics()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_metrics(self, client: Worqhat) -> None:
        with client.flows.with_streaming_response.get_metrics() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_trigger_with_file(self, client: Worqhat) -> None:
        flow = client.flows.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
        )
        assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_trigger_with_file_with_all_params(self, client: Worqhat) -> None:
        flow = client.flows.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            file=b"raw file contents",
            url="https://example.com/path/to/file.pdf",
        )
        assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_trigger_with_file(self, client: Worqhat) -> None:
        response = client.flows.with_raw_response.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_trigger_with_file(self, client: Worqhat) -> None:
        with client.flows.with_streaming_response.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_trigger_with_file(self, client: Worqhat) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            client.flows.with_raw_response.trigger_with_file(
                flow_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_trigger_with_payload(self, client: Worqhat) -> None:
        flow = client.flows.trigger_with_payload(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            body={
                "key1": "bar",
                "key2": "bar",
            },
        )
        assert_matches_type(FlowTriggerWithPayloadResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_trigger_with_payload(self, client: Worqhat) -> None:
        response = client.flows.with_raw_response.trigger_with_payload(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            body={
                "key1": "bar",
                "key2": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowTriggerWithPayloadResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_trigger_with_payload(self, client: Worqhat) -> None:
        with client.flows.with_streaming_response.trigger_with_payload(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            body={
                "key1": "bar",
                "key2": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowTriggerWithPayloadResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_trigger_with_payload(self, client: Worqhat) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            client.flows.with_raw_response.trigger_with_payload(
                flow_id="",
                body={
                    "key1": "bar",
                    "key2": "bar",
                },
            )


class TestAsyncFlows:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_metrics(self, async_client: AsyncWorqhat) -> None:
        flow = await async_client.flows.get_metrics()
        assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_metrics_with_all_params(self, async_client: AsyncWorqhat) -> None:
        flow = await async_client.flows.get_metrics(
            end_date=parse_date("2025-07-24"),
            start_date=parse_date("2025-07-01"),
            status="completed",
            user_id="member-test-2f9b9a4f-5898-4e7a-8f26-e60cea49ae31",
        )
        assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_metrics(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.flows.with_raw_response.get_metrics()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_metrics(self, async_client: AsyncWorqhat) -> None:
        async with async_client.flows.with_streaming_response.get_metrics() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowGetMetricsResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_trigger_with_file(self, async_client: AsyncWorqhat) -> None:
        flow = await async_client.flows.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
        )
        assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_trigger_with_file_with_all_params(self, async_client: AsyncWorqhat) -> None:
        flow = await async_client.flows.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            file=b"raw file contents",
            url="https://example.com/path/to/file.pdf",
        )
        assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_trigger_with_file(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.flows.with_raw_response.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_trigger_with_file(self, async_client: AsyncWorqhat) -> None:
        async with async_client.flows.with_streaming_response.trigger_with_file(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowTriggerWithFileResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_trigger_with_file(self, async_client: AsyncWorqhat) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            await async_client.flows.with_raw_response.trigger_with_file(
                flow_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_trigger_with_payload(self, async_client: AsyncWorqhat) -> None:
        flow = await async_client.flows.trigger_with_payload(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            body={
                "key1": "bar",
                "key2": "bar",
            },
        )
        assert_matches_type(FlowTriggerWithPayloadResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_trigger_with_payload(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.flows.with_raw_response.trigger_with_payload(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            body={
                "key1": "bar",
                "key2": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowTriggerWithPayloadResponse, flow, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_trigger_with_payload(self, async_client: AsyncWorqhat) -> None:
        async with async_client.flows.with_streaming_response.trigger_with_payload(
            flow_id="f825ab82-371f-40cb-9bed-b325531ead4a",
            body={
                "key1": "bar",
                "key2": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowTriggerWithPayloadResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_trigger_with_payload(self, async_client: AsyncWorqhat) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            await async_client.flows.with_raw_response.trigger_with_payload(
                flow_id="",
                body={
                    "key1": "bar",
                    "key2": "bar",
                },
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DBInsertRecordResponse"]


class DBInsertRecordResponse(BaseModel):
    data: Optional[object] = None

    message: Optional[str] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["FlowGetMetricsParams"]


class FlowGetMetricsParams(TypedDict, total=False):
    end_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """End date for filtering (YYYY-MM-DD format)"""

    start_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """Start date for filtering (YYYY-MM-DD format)"""

    status: Literal["completed", "failed", "in_progress"]
    """Filter by workflow status"""

    user_id: str
    """Filter by specific user ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .health_check_response import HealthCheckResponse as HealthCheckResponse
from .db_execute_query_params import DBExecuteQueryParams as DBExecuteQueryParams
from .db_insert_record_params import DBInsertRecordParams as DBInsertRecordParams
from .flow_get_metrics_params import FlowGetMetricsParams as FlowGetMetricsParams
from .db_delete_records_params import DBDeleteRecordsParams as DBDeleteRecordsParams
from .db_update_records_params import DBUpdateRecordsParams as DBUpdateRecordsParams
from .get_server_info_response import GetServerInfoResponse as GetServerInfoResponse
from .db_execute_query_response import DBExecuteQueryResponse as DBExecuteQueryResponse
from .db_insert_record_response import DBInsertRecordResponse as DBInsertRecordResponse
from .flow_get_metrics_response import FlowGetMetricsResponse as FlowGetMetricsResponse
from .db_delete_records_response import DBDeleteRecordsResponse as DBDeleteRecordsResponse
from .db_process_nl_query_params import DBProcessNlQueryParams as DBProcessNlQueryParams
from .db_update_records_response import DBUpdateRecordsResponse as DBUpdateRecordsResponse
from .db_process_nl_query_response import DBProcessNlQueryResponse as DBProcessNlQueryResponse
from .flow_trigger_with_file_params import FlowTriggerWithFileParams as FlowTriggerWithFileParams
from .flow_trigger_with_file_response import FlowTriggerWithFileResponse as FlowTriggerWithFileResponse
from .flow_trigger_with_payload_params import FlowTriggerWithPayloadParams as FlowTriggerWithPayloadParams
from .flow_trigger_with_payload_response import FlowTriggerWithPayloadResponse as FlowTriggerWithPayloadResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["FlowTriggerWithPayloadResponse"]


class FlowTriggerWithPayloadResponse(BaseModel):
    analytics_id: Optional[str] = None

    data: Optional[Dict[str, object]] = None

    message: Optional[str] = None

    success: Optional[bool] = None

    timestamp: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import FileTypes

__all__ = ["FlowTriggerWithFileParams"]


class FlowTriggerWithFileParams(TypedDict, total=False):
    file: FileTypes
    """File to upload and process"""

    url: str
    """URL to a file to download and process"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from worqhat import Worqhat, AsyncWorqhat
from tests.utils import assert_matches_type
from worqhat.types import (
    DBExecuteQueryResponse,
    DBInsertRecordResponse,
    DBDeleteRecordsResponse,
    DBUpdateRecordsResponse,
    DBProcessNlQueryResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDB:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_records(self, client: Worqhat) -> None:
        db = client.db.delete_records(
            table="users",
            where={"id": "123"},
        )
        assert_matches_type(DBDeleteRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete_records(self, client: Worqhat) -> None:
        response = client.db.with_raw_response.delete_records(
            table="users",
            where={"id": "123"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = response.parse()
        assert_matches_type(DBDeleteRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete_records(self, client: Worqhat) -> None:
        with client.db.with_streaming_response.delete_records(
            table="users",
            where={"id": "123"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = response.parse()
            assert_matches_type(DBDeleteRecordsResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_execute_query(self, client: Worqhat) -> None:
        db = client.db.execute_query(
            query="SELECT * FROM users WHERE status = 'active' LIMIT 10",
        )
        assert_matches_type(DBExecuteQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_execute_query(self, client: Worqhat) -> None:
        response = client.db.with_raw_response.execute_query(
            query="SELECT * FROM users WHERE status = 'active' LIMIT 10",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = response.parse()
        assert_matches_type(DBExecuteQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_execute_query(self, client: Worqhat) -> None:
        with client.db.with_streaming_response.execute_query(
            query="SELECT * FROM users WHERE status = 'active' LIMIT 10",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = response.parse()
            assert_matches_type(DBExecuteQueryResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_insert_record(self, client: Worqhat) -> None:
        db = client.db.insert_record(
            data={
                "name": "John Doe",
                "email": "john@worqhat.com",
            },
            table="users",
        )
        assert_matches_type(DBInsertRecordResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_insert_record(self, client: Worqhat) -> None:
        response = client.db.with_raw_response.insert_record(
            data={
                "name": "John Doe",
                "email": "john@worqhat.com",
            },
            table="users",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = response.parse()
        assert_matches_type(DBInsertRecordResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_insert_record(self, client: Worqhat) -> None:
        with client.db.with_streaming_response.insert_record(
            data={
                "name": "John Doe",
                "email": "john@worqhat.com",
            },
            table="users",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = response.parse()
            assert_matches_type(DBInsertRecordResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_process_nl_query(self, client: Worqhat) -> None:
        db = client.db.process_nl_query(
            question="How many active users do we have?",
        )
        assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_process_nl_query_with_all_params(self, client: Worqhat) -> None:
        db = client.db.process_nl_query(
            question="How many active users do we have?",
            context={"timeframe": "last 30 days"},
        )
        assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_process_nl_query(self, client: Worqhat) -> None:
        response = client.db.with_raw_response.process_nl_query(
            question="How many active users do we have?",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = response.parse()
        assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_process_nl_query(self, client: Worqhat) -> None:
        with client.db.with_streaming_response.process_nl_query(
            question="How many active users do we have?",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = response.parse()
            assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update_records(self, client: Worqhat) -> None:
        db = client.db.update_records(
            data={"status": "active"},
            table="users",
            where={"id": "123"},
        )
        assert_matches_type(DBUpdateRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_records(self, client: Worqhat) -> None:
        response = client.db.with_raw_response.update_records(
            data={"status": "active"},
            table="users",
            where={"id": "123"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = response.parse()
        assert_matches_type(DBUpdateRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_records(self, client: Worqhat) -> None:
        with client.db.with_streaming_response.update_records(
            data={"status": "active"},
            table="users",
            where={"id": "123"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = response.parse()
            assert_matches_type(DBUpdateRecordsResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDB:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_records(self, async_client: AsyncWorqhat) -> None:
        db = await async_client.db.delete_records(
            table="users",
            where={"id": "123"},
        )
        assert_matches_type(DBDeleteRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete_records(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.db.with_raw_response.delete_records(
            table="users",
            where={"id": "123"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = await response.parse()
        assert_matches_type(DBDeleteRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete_records(self, async_client: AsyncWorqhat) -> None:
        async with async_client.db.with_streaming_response.delete_records(
            table="users",
            where={"id": "123"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = await response.parse()
            assert_matches_type(DBDeleteRecordsResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_execute_query(self, async_client: AsyncWorqhat) -> None:
        db = await async_client.db.execute_query(
            query="SELECT * FROM users WHERE status = 'active' LIMIT 10",
        )
        assert_matches_type(DBExecuteQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_execute_query(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.db.with_raw_response.execute_query(
            query="SELECT * FROM users WHERE status = 'active' LIMIT 10",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = await response.parse()
        assert_matches_type(DBExecuteQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_execute_query(self, async_client: AsyncWorqhat) -> None:
        async with async_client.db.with_streaming_response.execute_query(
            query="SELECT * FROM users WHERE status = 'active' LIMIT 10",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = await response.parse()
            assert_matches_type(DBExecuteQueryResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_insert_record(self, async_client: AsyncWorqhat) -> None:
        db = await async_client.db.insert_record(
            data={
                "name": "John Doe",
                "email": "john@worqhat.com",
            },
            table="users",
        )
        assert_matches_type(DBInsertRecordResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_insert_record(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.db.with_raw_response.insert_record(
            data={
                "name": "John Doe",
                "email": "john@worqhat.com",
            },
            table="users",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = await response.parse()
        assert_matches_type(DBInsertRecordResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_insert_record(self, async_client: AsyncWorqhat) -> None:
        async with async_client.db.with_streaming_response.insert_record(
            data={
                "name": "John Doe",
                "email": "john@worqhat.com",
            },
            table="users",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = await response.parse()
            assert_matches_type(DBInsertRecordResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_process_nl_query(self, async_client: AsyncWorqhat) -> None:
        db = await async_client.db.process_nl_query(
            question="How many active users do we have?",
        )
        assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_process_nl_query_with_all_params(self, async_client: AsyncWorqhat) -> None:
        db = await async_client.db.process_nl_query(
            question="How many active users do we have?",
            context={"timeframe": "last 30 days"},
        )
        assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_process_nl_query(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.db.with_raw_response.process_nl_query(
            question="How many active users do we have?",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = await response.parse()
        assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_process_nl_query(self, async_client: AsyncWorqhat) -> None:
        async with async_client.db.with_streaming_response.process_nl_query(
            question="How many active users do we have?",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = await response.parse()
            assert_matches_type(DBProcessNlQueryResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_records(self, async_client: AsyncWorqhat) -> None:
        db = await async_client.db.update_records(
            data={"status": "active"},
            table="users",
            where={"id": "123"},
        )
        assert_matches_type(DBUpdateRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_records(self, async_client: AsyncWorqhat) -> None:
        response = await async_client.db.with_raw_response.update_records(
            data={"status": "active"},
            table="users",
            where={"id": "123"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        db = await response.parse()
        assert_matches_type(DBUpdateRecordsResponse, db, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_records(self, async_client: AsyncWorqhat) -> None:
        async with async_client.db.with_streaming_response.update_records(
            data={"status": "active"},
            table="users",
            where={"id": "123"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            db = await response.parse()
            assert_matches_type(DBUpdateRecordsResponse, db, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    db_execute_query_params,
    db_insert_record_params,
    db_delete_records_params,
    db_update_records_params,
    db_process_nl_query_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.db_execute_query_response import DBExecuteQueryResponse
from ..types.db_insert_record_response import DBInsertRecordResponse
from ..types.db_delete_records_response import DBDeleteRecordsResponse
from ..types.db_update_records_response import DBUpdateRecordsResponse
from ..types.db_process_nl_query_response import DBProcessNlQueryResponse

__all__ = ["DBResource", "AsyncDBResource"]


class DBResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DBResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#accessing-raw-response-data-eg-headers
        """
        return DBResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DBResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#with_streaming_response
        """
        return DBResourceWithStreamingResponse(self)

    def delete_records(
        self,
        *,
        table: str,
        where: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBDeleteRecordsResponse:
        """
        Deletes records from the specified table that match the where conditions.
        Organization ID filtering is automatically applied for multi-tenant security.

        Args:
          table: Table name to delete from

          where: Where conditions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/db/delete",
            body=maybe_transform(
                {
                    "table": table,
                    "where": where,
                },
                db_delete_records_params.DBDeleteRecordsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBDeleteRecordsResponse,
        )

    def execute_query(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBExecuteQueryResponse:
        """Executes a raw SQL query directly against ClickHouse (WorqDB).

        This endpoint
        provides direct SQL access with security guardrails to prevent destructive
        operations.

        Args:
          query: SQL query to execute

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/db/query",
            body=maybe_transform({"query": query}, db_execute_query_params.DBExecuteQueryParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBExecuteQueryResponse,
        )

    def insert_record(
        self,
        *,
        data: object,
        table: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBInsertRecordResponse:
        """Inserts a new record into the specified table.

        Organization ID is automatically
        added for multi-tenant security.

        Args:
          data: Data to insert

          table: Table name to insert into

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/db/insert",
            body=maybe_transform(
                {
                    "data": data,
                    "table": table,
                },
                db_insert_record_params.DBInsertRecordParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBInsertRecordResponse,
        )

    def process_nl_query(
        self,
        *,
        question: str,
        context: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBProcessNlQueryResponse:
        """
        Converts a natural language question into a SQL query and executes it.
        Organization ID filtering is automatically applied for multi-tenant security.

        Args:
          question: Natural language question

          context: Additional context for the query

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/db/nl-query",
            body=maybe_transform(
                {
                    "question": question,
                    "context": context,
                },
                db_process_nl_query_params.DBProcessNlQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBProcessNlQueryResponse,
        )

    def update_records(
        self,
        *,
        data: object,
        table: str,
        where: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBUpdateRecordsResponse:
        """
        Updates records in the specified table that match the where conditions.
        Organization ID filtering is automatically applied for multi-tenant security.

        Args:
          data: Data to update

          table: Table name to update

          where: Where conditions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/db/update",
            body=maybe_transform(
                {
                    "data": data,
                    "table": table,
                    "where": where,
                },
                db_update_records_params.DBUpdateRecordsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBUpdateRecordsResponse,
        )


class AsyncDBResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDBResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDBResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDBResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#with_streaming_response
        """
        return AsyncDBResourceWithStreamingResponse(self)

    async def delete_records(
        self,
        *,
        table: str,
        where: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBDeleteRecordsResponse:
        """
        Deletes records from the specified table that match the where conditions.
        Organization ID filtering is automatically applied for multi-tenant security.

        Args:
          table: Table name to delete from

          where: Where conditions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/db/delete",
            body=await async_maybe_transform(
                {
                    "table": table,
                    "where": where,
                },
                db_delete_records_params.DBDeleteRecordsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBDeleteRecordsResponse,
        )

    async def execute_query(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBExecuteQueryResponse:
        """Executes a raw SQL query directly against ClickHouse (WorqDB).

        This endpoint
        provides direct SQL access with security guardrails to prevent destructive
        operations.

        Args:
          query: SQL query to execute

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/db/query",
            body=await async_maybe_transform({"query": query}, db_execute_query_params.DBExecuteQueryParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBExecuteQueryResponse,
        )

    async def insert_record(
        self,
        *,
        data: object,
        table: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBInsertRecordResponse:
        """Inserts a new record into the specified table.

        Organization ID is automatically
        added for multi-tenant security.

        Args:
          data: Data to insert

          table: Table name to insert into

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/db/insert",
            body=await async_maybe_transform(
                {
                    "data": data,
                    "table": table,
                },
                db_insert_record_params.DBInsertRecordParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBInsertRecordResponse,
        )

    async def process_nl_query(
        self,
        *,
        question: str,
        context: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBProcessNlQueryResponse:
        """
        Converts a natural language question into a SQL query and executes it.
        Organization ID filtering is automatically applied for multi-tenant security.

        Args:
          question: Natural language question

          context: Additional context for the query

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/db/nl-query",
            body=await async_maybe_transform(
                {
                    "question": question,
                    "context": context,
                },
                db_process_nl_query_params.DBProcessNlQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBProcessNlQueryResponse,
        )

    async def update_records(
        self,
        *,
        data: object,
        table: str,
        where: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DBUpdateRecordsResponse:
        """
        Updates records in the specified table that match the where conditions.
        Organization ID filtering is automatically applied for multi-tenant security.

        Args:
          data: Data to update

          table: Table name to update

          where: Where conditions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/db/update",
            body=await async_maybe_transform(
                {
                    "data": data,
                    "table": table,
                    "where": where,
                },
                db_update_records_params.DBUpdateRecordsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DBUpdateRecordsResponse,
        )


class DBResourceWithRawResponse:
    def __init__(self, db: DBResource) -> None:
        self._db = db

        self.delete_records = to_raw_response_wrapper(
            db.delete_records,
        )
        self.execute_query = to_raw_response_wrapper(
            db.execute_query,
        )
        self.insert_record = to_raw_response_wrapper(
            db.insert_record,
        )
        self.process_nl_query = to_raw_response_wrapper(
            db.process_nl_query,
        )
        self.update_records = to_raw_response_wrapper(
            db.update_records,
        )


class AsyncDBResourceWithRawResponse:
    def __init__(self, db: AsyncDBResource) -> None:
        self._db = db

        self.delete_records = async_to_raw_response_wrapper(
            db.delete_records,
        )
        self.execute_query = async_to_raw_response_wrapper(
            db.execute_query,
        )
        self.insert_record = async_to_raw_response_wrapper(
            db.insert_record,
        )
        self.process_nl_query = async_to_raw_response_wrapper(
            db.process_nl_query,
        )
        self.update_records = async_to_raw_response_wrapper(
            db.update_records,
        )


class DBResourceWithStreamingResponse:
    def __init__(self, db: DBResource) -> None:
        self._db = db

        self.delete_records = to_streamed_response_wrapper(
            db.delete_records,
        )
        self.execute_query = to_streamed_response_wrapper(
            db.execute_query,
        )
        self.insert_record = to_streamed_response_wrapper(
            db.insert_record,
        )
        self.process_nl_query = to_streamed_response_wrapper(
            db.process_nl_query,
        )
        self.update_records = to_streamed_response_wrapper(
            db.update_records,
        )


class AsyncDBResourceWithStreamingResponse:
    def __init__(self, db: AsyncDBResource) -> None:
        self._db = db

        self.delete_records = async_to_streamed_response_wrapper(
            db.delete_records,
        )
        self.execute_query = async_to_streamed_response_wrapper(
            db.execute_query,
        )
        self.insert_record = async_to_streamed_response_wrapper(
            db.insert_record,
        )
        self.process_nl_query = async_to_streamed_response_wrapper(
            db.process_nl_query,
        )
        self.update_records = async_to_streamed_response_wrapper(
            db.update_records,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["HealthCheckResponse", "Services"]


class Services(BaseModel):
    database: Optional[Literal["ok", "degraded", "down"]] = None


class HealthCheckResponse(BaseModel):
    status: Literal["ok", "degraded", "maintenance", "down"]
    """Current health status of the API"""

    uptime: float
    """Server uptime in seconds"""

    environment: Optional[Literal["development", "staging", "production"]] = None
    """Current environment"""

    services: Optional[Services] = None
    """Status of dependent services"""

    timestamp: Optional[datetime] = None
    """Current server time"""

    version: Optional[str] = None
    """Current API version"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Mapping, cast
from datetime import date
from typing_extensions import Literal

import httpx

from ..types import flow_get_metrics_params, flow_trigger_with_file_params, flow_trigger_with_payload_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.flow_get_metrics_response import FlowGetMetricsResponse
from ..types.flow_trigger_with_file_response import FlowTriggerWithFileResponse
from ..types.flow_trigger_with_payload_response import FlowTriggerWithPayloadResponse

__all__ = ["FlowsResource", "AsyncFlowsResource"]


class FlowsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#accessing-raw-response-data-eg-headers
        """
        return FlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#with_streaming_response
        """
        return FlowsResourceWithStreamingResponse(self)

    def get_metrics(
        self,
        *,
        end_date: Union[str, date] | NotGiven = NOT_GIVEN,
        start_date: Union[str, date] | NotGiven = NOT_GIVEN,
        status: Literal["completed", "failed", "in_progress"] | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FlowGetMetricsResponse:
        """Get metrics for workflows within a specified date range.

        This endpoint provides
        aggregated statistics about workflow execution and detailed information about
        individual workflows.

        The response includes metrics aggregated by user and a list of all workflows
        matching the specified criteria.

        Args:
          end_date: End date for filtering (YYYY-MM-DD format)

          start_date: Start date for filtering (YYYY-MM-DD format)

          status: Filter by workflow status

          user_id: Filter by specific user ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/flows/metrics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                        "status": status,
                        "user_id": user_id,
                    },
                    flow_get_metrics_params.FlowGetMetricsParams,
                ),
            ),
            cast_to=FlowGetMetricsResponse,
        )

    def trigger_with_file(
        self,
        flow_id: str,
        *,
        file: FileTypes | NotGiven = NOT_GIVEN,
        url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FlowTriggerWithFileResponse:
        """Trigger a workflow by its ID with either a file upload or a URL to a file.

        This
        endpoint accepts multipart/form-data with either a file field or a URL field.
        When a URL is provided, the server will download the file and process it as if
        it were directly uploaded. The workflow ID must be specified in the URL path.

        Args:
          file: File to upload and process

          url: URL to a file to download and process

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "url": url,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/flows/file/{flow_id}",
            body=maybe_transform(body, flow_trigger_with_file_params.FlowTriggerWithFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowTriggerWithFileResponse,
        )

    def trigger_with_payload(
        self,
        flow_id: str,
        *,
        body: Dict[str, object],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FlowTriggerWithPayloadResponse:
        """Trigger a workflow by its ID with a JSON payload.

        This endpoint accepts any
        valid JSON object as the request body and forwards it to the workflow. The
        workflow ID must be specified in the URL path.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return self._post(
            f"/flows/trigger/{flow_id}",
            body=maybe_transform(body, flow_trigger_with_payload_params.FlowTriggerWithPayloadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowTriggerWithPayloadResponse,
        )


class AsyncFlowsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncFlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/WorqHat/worqhat-python-sdk#with_streaming_response
        """
        return AsyncFlowsResourceWithStreamingResponse(self)

    async def get_metrics(
        self,
        *,
        end_date: Union[str, date] | NotGiven = NOT_GIVEN,
        start_date: Union[str, date] | NotGiven = NOT_GIVEN,
        status: Literal["completed", "failed", "in_progress"] | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FlowGetMetricsResponse:
        """Get metrics for workflows within a specified date range.

        This endpoint provides
        aggregated statistics about workflow execution and detailed information about
        individual workflows.

        The response includes metrics aggregated by user and a list of all workflows
        matching the specified criteria.

        Args:
          end_date: End date for filtering (YYYY-MM-DD format)

          start_date: Start date for filtering (YYYY-MM-DD format)

          status: Filter by workflow status

          user_id: Filter by specific user ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/flows/metrics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                        "status": status,
                        "user_id": user_id,
                    },
                    flow_get_metrics_params.FlowGetMetricsParams,
                ),
            ),
            cast_to=FlowGetMetricsResponse,
        )

    async def trigger_with_file(
        self,
        flow_id: str,
        *,
        file: FileTypes | NotGiven = NOT_GIVEN,
        url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FlowTriggerWithFileResponse:
        """Trigger a workflow by its ID with either a file upload or a URL to a file.

        This
        endpoint accepts multipart/form-data with either a file field or a URL field.
        When a URL is provided, the server will download the file and process it as if
        it were directly uploaded. The workflow ID must be specified in the URL path.

        Args:
          file: File to upload and process

          url: URL to a file to download and process

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "url": url,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/flows/file/{flow_id}",
            body=await async_maybe_transform(body, flow_trigger_with_file_params.FlowTriggerWithFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowTriggerWithFileResponse,
        )

    async def trigger_with_payload(
        self,
        flow_id: str,
        *,
        body: Dict[str, object],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FlowTriggerWithPayloadResponse:
        """Trigger a workflow by its ID with a JSON payload.

        This endpoint accepts any
        valid JSON object as the request body and forwards it to the workflow. The
        workflow ID must be specified in the URL path.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return await self._post(
            f"/flows/trigger/{flow_id}",
            body=await async_maybe_transform(body, flow_trigger_with_payload_params.FlowTriggerWithPayloadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowTriggerWithPayloadResponse,
        )


class FlowsResourceWithRawResponse:
    def __init__(self, flows: FlowsResource) -> None:
        self._flows = flows

        self.get_metrics = to_raw_response_wrapper(
            flows.get_metrics,
        )
        self.trigger_with_file = to_raw_response_wrapper(
            flows.trigger_with_file,
        )
        self.trigger_with_payload = to_raw_response_wrapper(
            flows.trigger_with_payload,
        )


class AsyncFlowsResourceWithRawResponse:
    def __init__(self, flows: AsyncFlowsResource) -> None:
        self._flows = flows

        self.get_metrics = async_to_raw_response_wrapper(
            flows.get_metrics,
        )
        self.trigger_with_file = async_to_raw_response_wrapper(
            flows.trigger_with_file,
        )
        self.trigger_with_payload = async_to_raw_response_wrapper(
            flows.trigger_with_payload,
        )


class FlowsResourceWithStreamingResponse:
    def __init__(self, flows: FlowsResource) -> None:
        self._flows = flows

        self.get_metrics = to_streamed_response_wrapper(
            flows.get_metrics,
        )
        self.trigger_with_file = to_streamed_response_wrapper(
            flows.trigger_with_file,
        )
        self.trigger_with_payload = to_streamed_response_wrapper(
            flows.trigger_with_payload,
        )


class AsyncFlowsResourceWithStreamingResponse:
    def __init__(self, flows: AsyncFlowsResource) -> None:
        self._flows = flows

        self.get_metrics = async_to_streamed_response_wrapper(
            flows.get_metrics,
        )
        self.trigger_with_file = async_to_streamed_response_wrapper(
            flows.trigger_with_file,
        )
        self.trigger_with_payload = async_to_streamed_response_wrapper(
            flows.trigger_with_payload,
        )
